const util = require('../src/partition.js')

describe('filterSuite plugin', () => {
  describe('create partitions', () => {
    const list = [
      { fullTitle: 'test 1', duration: 10000 },
      { fullTitle: 'test 2', duration: 9000 },
      { fullTitle: 'test 3', duration: 8000 },
      { fullTitle: 'test 4', duration: 7000 },
      { fullTitle: 'test 5', duration: 6000 },
      { fullTitle: 'test 6', duration: 6000 },
      { fullTitle: 'test 7', duration: 7000 },
      { fullTitle: 'test 8', duration: 8000 },
      { fullTitle: 'test 9', duration: 9000 },
      { fullTitle: 'test 10', duration: 10000 }
    ]
    it('should return empty list if list is empty', () => {
      expect(util.partition([], 1)).to.deep.equal([])
    })
    it('should return one list with all titles if nodeTotal is 1', () => {
      expect(util.partition(list, 1)).to.deep.equal([list.map(t => t.fullTitle)])
    })
    it('should return partitions of size 1 if nodeTotal is equal to list size', () => {
      const results = util.partition(list, 10)
      expect(results).to.have.length(10)
      for (const partition of results) {
        expect(partition).to.have.length(1)
        expect(partition[0]).to.be.oneOf(list.map(t => t.fullTitle))
      }
    })
    it('should return partitions of size 1 if nodeTotal is bigger than list', () => {
      const results = util.partition(list, 11)
      expect(results).to.have.length(10)
      for (const partition of results) {
        expect(partition).to.have.length(1)
        expect(partition[0]).to.be.oneOf(list.map(t => t.fullTitle))
      }
    })
    it('should fill up partitions with short tests as long as possible', () => {
      const results = util.partition([
        { fullTitle: 'test 1', duration: 1 },
        { fullTitle: 'test 2', duration: 1 },
        { fullTitle: 'test 3', duration: 1 },
        { fullTitle: 'test 10', duration: 3 }
      ], 10)
      expect(results).to.have.length(2)
      expect(results.find(p => p.includes('test 10'))).to.have.length(1)
    })
    it('should partition thousands of tests', () => {
      const list = new Array(10000).fill(0).map((_, i) => ({
        fullTitle: `test ${i}`,
        duration: Math.floor(Math.random() * 10000)
      }))
      const result = util.partition(list, 10)
      expect(result).to.have.length(10)
      result.map(p => Math.abs(1000 - p.length)).forEach(difference => {
        // expect partition sizes to be within 100 tests of each other
        expect(difference).to.be.below(100)
      })
    })
  })
})
