import { describe, it, expect, afterEach } from '@jest/globals'
import { build, createServer } from 'vite'
import vitePluginOxExternals from '../../src/index'
import { getFileFromBundle, getPort, getDirname } from '../util'

const PORT = getPort()
const __dirname = getDirname(import.meta.url)

describe('Custom prefix', function () {
  let server

  afterEach(async function () {
    if (server) {
      await server.close()
      await server.httpServer.close()
      server = null
    }
  })

  it('works', async function () {
    const bundle = await build({
      root: __dirname,
      logLevel: 'silent',
      build: {
        write: false,
        rollupOptions: {
          output: {
            entryFileNames: 'entries/[name].js'
          }
        }
      },
      plugins: [vitePluginOxExternals({
        prefix: '~'
      })]
    })

    const indexJS = getFileFromBundle('index.js', bundle)
    expect(indexJS.code).toContain('"../module/external.js"')
    expect(indexJS.code).toContain('"../external.js"')
    expect(indexJS.code).toContain('No replace: import mod from "~/module/external"')
    expect(indexJS.imports).toEqual(['module/external.js', 'external.js'])
  })

  it('works in dev mode', async function () {
    server = await createServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginOxExternals({
        prefix: '~'
      })]
    })
    await server.listen(PORT)

    const res = await fetch(`http://localhost:${PORT}/index.js`)
    // Since vite dev server injects inline source maps, we can't use strict equality here
    expect(await res.text()).toEqual(expect.stringContaining('import mod from "/module/external.js"\nimport "/external.js"\n\nconsole.log(\'No replace: import mod from "~/module/external"\', mod)\n'))

    await server.close()
  })
})
